import CardapioInteligente from './CardapioInteligente.js';
import Algoritmo from './Algoritmo.js';
import Optimizer from './Optimizer.js';
import Status from './Status.js';
import axios from 'axios';
import BD from '../connection.js';
import RequestExceptionHandler from '../Exceptions/RequestExceptionHandler.js';

const OtimizadorLote = {
    async testConnection(){
        const result = await CardapioInteligente.testConnection();
        return result;
    },
    async getParametersGenData( params ){
        const { NRGERACAOCARD, NRITGERACAOCARD, CDOPERADOR, NRORG } = params;
        const backOfficeData = await CardapioInteligente.getBackOfficeData(NRGERACAOCARD, NRITGERACAOCARD);
        let paramsGetGenData = {
            IDTIPORESTRICAO: backOfficeData.IDTIPORESTRICAO,
            DTREFCUSPADR   : backOfficeData.DTMESCARD,
            DTMES          : backOfficeData.DTMESCARD,
            DIAS_CONG      : backOfficeData.QTDIASCONG,
            VLR_CONGELADO  : backOfficeData.QTDIASCONG,
            CDNVPRATO      : backOfficeData.CDPICTPRAT.match(/\./g).length+1,
            CDOPERADOR,
            NRORG
        };
        return Object.assign({}, params, paramsGetGenData, backOfficeData);
    },
    async isValidBatchItem( NRGERACAOCARD, NRITGERACAOCARD, NRORG, ROW_ID, socket ){
        const IS_VALID = await CardapioInteligente.isValidServiceLink(NRGERACAOCARD, NRITGERACAOCARD);
        if(!IS_VALID){
            const deleteResponse = await CardapioInteligente.deleteOldLog( NRITGERACAOCARD );
            const ERROR = 'Serviço possui tipos de pratos vinculados mas não possui cenário vinculado informado.';
            const result = await CardapioInteligente.insereLog({
                NRITGERACAOCARD,
                IDTIPOLOG: 'UNEXPECTED_ERROR',
                DSLOGGERACAOCARD: ERROR,
                NRORG
            });
            const response = {
                ROW_ID,
                STATUS: 'ERRO',
                data: ERROR,
                OPERATION: "GENERATE",
                NRITGERACAOCARD,
                NRCENARIOCARD: null
            };
            let responseUpdateStatus = await Status.updateStatusItGeracaoCard(NRITGERACAOCARD, response.STATUS);
            socket.emit('onSocketSceneryBatchMessage', response);
        }
        return IS_VALID;
    },
    isValidCalendar( data ){
        return data && data.calendario && data.calendario.data && data.calendario.data.length;
    },
    removeReceitaPratoIfNotFrequencia( dataSet ){
        if(dataSet.frq_grupo_prato_mes.data.length) {
            dataSet.receita_prato.data = dataSet.receita_prato.data.filter(p => dataSet.frq_grupo_prato_mes.data.find(f => f.NUM_GRP_PRATO == p.NUM_GRP_PRATO));
        }
    },
    handleAlgorithmError(NRITGERACAOCARD, err, paramsAlgoritmo){
        let error = null;
        if(err && err.response && err.response.data){
            error = err.response.data;
            if(typeof error == 'object' && error.title === "ERRO_EXECUCAO"){
                error.DSLOGINTERNO = err.response.data.otmmessage;
                error.otmmessage = "Ocorreu um erro inesperado na execução do algoritmo";
                error.algorithmReDispatch = false;
            } else if(typeof error == 'string'){
                let errorMessage = error;
                error = {
                    algorithmReDispatch: false,
                    success: false,
                    otmmessage: `Erro ao chamar o algoritimo de otimização ${paramsAlgoritmo.url}`,
                    stack: errorMessage,
                    DSLOGITERNO: errorMessage
                };
            }
            error.title = "UNEXPECTED_ERROR";
        } else if(err && err.message){
            console.log(`ITEM: ${NRITGERACAOCARD}`, err.message);
            error = {
                algorithmReDispatch: true,
                success:false,
                otmmessage: `Erro ao chamar o algoritimo de otimização ${paramsAlgoritmo.url}`,
                title: "UNEXPECTED_ERROR",
                DSLOGINTERNO: err.message
            };
        } else {
            console.log(`ITEM: ${NRITGERACAOCARD} -- `, err);
            error = {
                algorithmReDispatch: true,
                success: false,
                otmmessage: `Erro ao chamar o algoritimo de otimização ${paramsAlgoritmo.url}`,
                title: "UNEXPECTED_ERROR",
                DSLOGINTERNO: err 
            }
        }
        return error;
    },
    async algorithm( NRITGERACAOCARD, paramsAlgoritmo ){
        let response = null;
        try {
            response = await Algoritmo.call(paramsAlgoritmo).catch(function( error ) {
                return {err: this.handleAlgorithmError(NRITGERACAOCARD, error, paramsAlgoritmo)};
            }.bind(this)).then( response => response);
        } catch(err){
            response = {err};
        }
        return response;
    },
    createPromise(){
        let resolve,reject;
        let promise = new Promise((res, rej)=>{resolve=res;reject=rej});
        return {
            promise,
            resolve,
            reject
        };
    },
    async promiseState(p) {
        const t = {};
        return Promise.race([p, t])
            .then(v => (v === t)? "pending" : "fulfilled", () => "rejected");
    },
    async attemptsToCallAlgorithm(NRITGERACAOCARD, paramsAlgorithm){
        const SECONDS = 70;
        const MILLISECONDS = SECONDS * 1000;
        const ATTEMPTS_TIME = MILLISECONDS;
        const MAX_ATTEMPTS = 2;
        let attempts = 1;
        let resolveP, rejectP;
        let promise = new Promise((res, rej)=>{resolveP = res; rejectP = rej});
        const firstResponse = await this.algorithm( NRITGERACAOCARD, paramsAlgorithm );
        let lastAttempt = Promise.resolve();
        if(firstResponse && firstResponse.err && firstResponse.err.algorithmReDispatch && attempts < MAX_ATTEMPTS) {
            let interval = setInterval(async function() {
                let status = await this.promiseState(lastAttempt);
                if(status === 'pending') return;
                let newPromise = this.createPromise();
                lastAttempt = newPromise.promise;
                console.log(`ITEM: ${NRITGERACAOCARD} -- Algoritmo(${attempts+1}): ${paramsAlgorithm.url}`);
                const response = await this.algorithm( NRITGERACAOCARD, paramsAlgorithm );
                newPromise.resolve();
                attempts++;
                if(response && response.err){
                    let error = this.handleAlgorithmError(NRITGERACAOCARD, response.err, paramsAlgorithm);
                    if(!error.algorithmReDispatch || attempts >= MAX_ATTEMPTS) {
                        clearInterval(interval);
                        resolveP(response);
                    }
                } else {
                    clearInterval(interval);
                    resolveP(response);
                }
            }.bind(this), ATTEMPTS_TIME);      
        } else {
            resolveP(firstResponse);
        }
        return promise;
    },

    async callBatchOptimization( params, socket ){
        let url = process.env.OTM_LOTE_HOST;
        if(process.env.OTM_LOTE_PORT){
            url = `${url}:${process.env.OTM_LOTE_PORT}`;
        }
        let paramsAxios = {
            method: 'POST',
            url: `${url}/optimizer`,
            headers: {'Content-Type': 'application/json'},
            data: params
        };
        const result = await axios(paramsAxios).catch( error => ({error}));
        if(result.error){
            let error = RequestExceptionHandler.getErrorMessage(result.error);
            console.log(`ITEM: ${params.NRITGERACAOCARD} -- ERROR CALL LINKED_ITEM`, error);
            await CardapioInteligente.deleteOldLog( params.NRITGERACAOCARD );
            await CardapioInteligente.insereLog({
                NRITGERACAOCARD: params.NRITGERACAOCARD,
                IDTIPOLOG: 'UNEXPECTED_ERROR',
                DSLOGGERACAOCARD: `Ocorreu um erro ao se conectar no servidor: ${url}`,
                DSLOGINTERNO: typeof error === 'string' ? error : 'UNHANDLED_ERROR'
            });
            await Status.updateStatusItGeracaoCard(params.NRITGERACAOCARD, 'ERRO');
            socket.emit('onSocketSceneryBatchMessage', {
                ROW_ID: params.ROW_ID,
                STATUS: 'ERRO',
                data: error,
                OPERATION: "GENERATE",
                NRITGERACAOCARD: params.NRITGERACAOCARD
            });
        }
        return result;
    },
    async updateLinkedItems(NRCENARIOCARD, linkedItems){
        try {
            let itemsConcat = linkedItems.map(Number).join(`','`); // ['123', '0012'] => (string) 123','12
            itemsConcat = `'${itemsConcat}'`;
            let isValid = itemsConcat.match(/[^\d\,\']/gi) == null; //patern match char different of [0-9 comma single quotes]
            if(!isValid) return "INVALID_ITEMS";
            const SQL = `UPDATE ITGERACAOCARD SET NRCENARIOVINCULO = :P_NRCENARIOVINCULO WHERE NRITGERACAOCARD IN ( ${itemsConcat} )`;
            const result = await BD.execSQL(SQL, Object.assign(
                BD.createParam('P_NRCENARIOVINCULO', NRCENARIOCARD)
            ));
        } catch( error ) {
            return ({error});
        }
    },
    async callBatchOptimizationForChildren(CHILDREN, $token, socket, NRGERACAOCARD, NRCENARIOCARD = null){
        try {
            if(CHILDREN && CHILDREN.length){
                await this.updateLinkedItems(NRCENARIOCARD, CHILDREN.map( item => item.NRITGERACAOCARD)).catch(error => console.log(error));
                CHILDREN.forEach( item => {
                    item.$token = $token;
                    item.NRCENARIOVINCULO = NRCENARIOCARD;
                    if( item.ROW_ID ) {
                        this.callBatchOptimization(item, socket);
                    } else {
                        socket.emit('onSocketSceneryBatchMessage', {
                            ROW_ID: null,
                            NRGERACAOCARD,
                            NRITGERACAOCARD: item.NRITGERACAOCARD,
                            NRCENARIOVINCULO: item.NRCENARIOVINCULO
                        });
                    }
                });
            }
            return {};
        } catch ( error ) {
            console.log(error);
            return ({error});
        }
    },
    async otimizar( requestParams, socket ){
        const timeStart = new Date().getTime();
        const { 
            CDOPERADOR,       NRORG,   NRGERACAOCARD, 
            NRITGERACAOCARD,  ROW_ID,  CHILDREN, 
            NRCENARIOVINCULO, UTL_GRP_FORA_VIG = 'N' 
        } = requestParams;
        const IS_VALID_BATCH_ITEM = await this.isValidBatchItem(NRGERACAOCARD, NRITGERACAOCARD, NRORG, ROW_ID, socket);
        if( !IS_VALID_BATCH_ITEM ) {
            const responseItGeracaoCard = await CardapioInteligente.updateItGeracaoCard(null, NRITGERACAOCARD);
            return null;
        }
        const params = await this.getParametersGenData( requestParams );
        const dataSet = await CardapioInteligente.retDadosCardInteligente( params );
        if(params.parsedCardapio){
            dataSet.parsedCardapio = params.parsedCardapio;
        }
        if(!this.isValidCalendar(dataSet)){
            let responseUpdateStatus = await Status.updateStatusItGeracaoCard(NRITGERACAOCARD, 'ERRO');
            const ERROR = "Unidade não possui comensais estimados ou está com calendário fechado.";
            const deleteResponse = await CardapioInteligente.deleteOldLog( NRITGERACAOCARD );
            const responseItGeracaoCard = await CardapioInteligente.updateItGeracaoCard(null, NRITGERACAOCARD);
            const result = await CardapioInteligente.insereLog({
                NRITGERACAOCARD,
                IDTIPOLOG: 'UNEXPECTED_ERROR',
                DSLOGGERACAOCARD: ERROR,
                NRORG
            });
            socket.emit('onSocketSceneryBatchMessage', {
                ROW_ID,
                STATUS: 'ERRO',
                data: ERROR,
                OPERATION: "GENERATE",
                NRITGERACAOCARD,
                NRCENARIOCARD: null
            });
            this.callBatchOptimizationForChildren( CHILDREN, requestParams.$token, socket, NRGERACAOCARD, null );
            return null;
        }
        CardapioInteligente.addIncidencia_tp_gp_prato(dataSet);
        const { VRCUSTPADR, VLR_CUSTO_MAX, VLR_CUSTO_MIN } = params;
        Object.assign(dataSet, {
            VRCUSTPADR, VLR_CUSTO_MAX, VLR_CUSTO_MIN, CDOPERADOR, NRORG, 
            P_DTMES      : params.DTMESCARD,
            P_VRCUSTPADR : VRCUSTPADR
        });
        if(UTL_GRP_FORA_VIG === "N") {
            this.removeReceitaPratoIfNotFrequencia( dataSet );
        }
        console.log(`ITEM: ${NRITGERACAOCARD} -- Algoritmo: ${process.env.HOST_ALGORITMO_SERVER}`);
        const paramsAlgoritmo = {
            url : process.env.HOST_ALGORITMO_SERVER,
            body: dataSet 
        };
        let error = null;
        const optimized = await this.attemptsToCallAlgorithm(NRITGERACAOCARD, paramsAlgoritmo);
        
        if(optimized.err){
            error = optimized.err;
        }
        let response = null;
        let promises = [];
        const deleteResponse = await CardapioInteligente.deleteOldLog( params.NRITGERACAOCARD );
        if( error ){
            error.IDTIPOLOG = error.title;
            error.DSLOGGERACAOCARD = error.otmmessage;
            response = {
                STATUS: 'ERRO',
                data: error,
                OPERATION: "GENERATE"
            };
            Object.assign(response, requestParams);
            let logParameters = Object.assign({}, requestParams, error);
            const result = await CardapioInteligente.insereLog(logParameters);
            const responseStatus = await Status.updateStatusItGeracaoCard(params.NRITGERACAOCARD, "ERRO");

            socket.emit('onSocketSceneryBatchMessage', response);
            this.callBatchOptimizationForChildren( CHILDREN, requestParams.$token, socket, NRGERACAOCARD, null );
        } else {
            const paramCenario = optimized.data.data;
            paramCenario.NRCENARIOVINCULO = params.NRCENARIOVINCULO;
            paramCenario.backOfficeData = params;
            Object.assign(paramCenario,  params);
            if(paramCenario.tabela && Array.isArray(paramCenario.tabela.pratos) && paramCenario.tabela.pratos.length) {
                paramCenario.tabela.pratos[0].forEach(tpprato => {
                    tpprato.Visivel = true;
                    tpprato.sequencial = tpprato.seq;
                });
            }
            paramCenario.freqData = Optimizer.feedFreqData( paramCenario );
            paramCenario.incidencia = dataSet.incidencia;
            paramCenario.JSONCARDAPIO = '{}';
            paramCenario.pu_aculm = paramCenario.pu_aculm 
                ? parseFloat(parseFloat(paramCenario.pu_aculm).toFixed(5))
                : 0;
            paramCenario.INGData = Optimizer.feedINGData( paramCenario );
            paramCenario.JSONCENARIO = JSON.stringify(paramCenario);
            promises.push(Algoritmo.checkGeneration(paramCenario));
            promises.push(CardapioInteligente.insereCenario(paramCenario));
            Promise.all(promises).then( async resp => {
                const [ LOGITGERACAOCARD, CENARIO ] = resp;
                const STATUS = LOGITGERACAOCARD.length ? "FINALIZADO_COM_ERRO" : "FINALIZADO_SEM_ERRO";
                const responseSocket = {
                    OPERATION: "GENERATE",
                    NRITGERACAOCARD: params.NRITGERACAOCARD, 
                    ROW_ID: params.ROW_ID, 
                    STATUS, 
                    NRCENARIOCARD: parseInt(CENARIO.NRCENARIOCARD), 
                    TIME: new Date().getTime() - timeStart
                };
                const responseStatus = await Status.updateStatusItGeracaoCard(params.NRITGERACAOCARD, STATUS);
                if(CHILDREN && CHILDREN.length){
                    await this.updateLinkedItems(responseSocket.NRCENARIOCARD, CHILDREN.map( item => item.NRITGERACAOCARD)).catch(error => console.log(error));
                    this.callBatchOptimizationForChildren( CHILDREN, requestParams.$token, socket, NRGERACAOCARD, responseSocket.NRCENARIOCARD );
                }
                if(NRCENARIOVINCULO !== null) {
                    responseSocket.NRCENARIOVINCULO = NRCENARIOVINCULO;
                }
                socket.emit('onSocketSceneryBatchMessage', responseSocket);
            }).catch( error => {
                socket.emit('onSocketSceneryBatchMessage', {
                    OPERATION: "GENERATE",
                    ROW_ID: params.ROW_ID,
                    STATUS: "ERRO",
                    NRITGERACAOCARD: params.NRITGERACAOCARD,
                    message: error.message, stack: error.stack
                });
                this.callBatchOptimizationForChildren( CHILDREN, requestParams.$token, socket, NRGERACAOCARD, null );
            });
        }
    }
};

export default OtimizadorLote;